# encoding: utf-8
class Gw::Admin::Webmail::FiltersController < Gw::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  layout "admin/gw/webmail"
  before_action :set_title

  def pre_dispatch
    imap = Core.imap
  end

  def index
    item = Gw::WebmailFilter.where(user_id: Core.user.id)
    item = item.order(sort_no: params[:sort], id: params[:sort]) if params[:sort].present?
    item = item.paginate(page: params[:page], per_page: params[:limit])
    @items = item.all
    _index @items
  end

  def show
    @item = Gw::WebmailFilter.find(params[:id])
    return error_auth unless @item.readable?
    _show @item
  end

  def new
    @item = Gw::WebmailFilter.new({
      state: 'enabled',
      conditions_chain: 'and',
      sort_no: 0
    })
  end

  def create
    @item = Gw::WebmailFilter.new(create_item_params)
    @item.user_id = Core.user.id
    @item.in_columns = params[:item][:in_columns]
    @item.in_inclusions = params[:item][:in_inclusions]
    @item.in_values = params[:item][:in_values]
    _create(@item)
  end

  def update
    @item = Gw::WebmailFilter.find(params[:id])
    return error_auth unless @item.editable?
    @item.attributes = update_item_params
    @item.user_id = Core.user.id
    @item.in_columns = params[:item][:in_columns]
    @item.in_inclusions = params[:item][:in_inclusions]
    @item.in_values = params[:item][:in_values]
    _update(@item)
  end

  def destroy
    @item = Gw::WebmailFilter.find(params[:id])
    _destroy(@item)
  end

  def apply
    @item = Gw::WebmailFilter.find(params[:id])
    return error_auth unless @item.readable?
    @f_item = Gw::WebmailFilter.new
    @f_item.attributes = apply_item_params if params[:f_item].present?
    return false unless request.post?

    ## validation
    @f_item.errors.add :base, I18n.t("helpers.message.folder") if @f_item.mailbox.blank?
    @f_item.errors.add :base, I18n.t("helpers.message.folder_cond")  if @item.conditions.size == 0
    return false if @f_item.errors.size > 0

    @applied = 0
    message  = ""
    changed_mailbox_uids = {}

    begin
      timeout = Sys::Lib::Timeout.new(60)
      if @f_item.include_sub == "1"
        mailboxes = Core.imap.list('', "#{@f_item.mailbox}.*") || []
        mailboxes.each do |mailbox|
          apply_mailbox(mailbox.name, timeout)
        end
      end
      apply_mailbox(@f_item.mailbox, timeout)
    rescue Sys::Lib::Timeout::Error => ex
      message = I18n.t("rumi.filter.message.timeout", second: ex.second)
    end

    case @item.action
    when 'move'
      changed_mailbox_uids[@item.mailbox] = [:all]
    when 'delete'
      changed_mailbox_uids['Trash'] = [:all]
    end

    Gw::WebmailMailbox.load_starred_mails(changed_mailbox_uids) if @applied > 0
    Gw::WebmailMailbox.load_mailboxes(reload = true) if @applied > 0
    flash[:notice] = I18n.t("helpers.message.mail_apli", message: message, count: @applied).html_safe
    redirect_to :action => :apply
  end
private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    @title = I18n.t('rumi.mail.title.filter')
  end

  def create_item_params
    params.require(:item).permit(:name,
                                 :state,
                                 :sort_no,
                                 :conditions_chain,
                                 :action,
                                 :mailbox)
  end

  def update_item_params
    params.require(:item).permit(:state,
                                 :sort_no, 
                                 :conditions_chain,
                                 :action,
                                 :mailbox)
  end

  def apply_item_params
    params.require(:f_item).permit(:mailbox, :include_sub)
  end

protected
  def apply_mailbox(mailbox, timeout)
    begin
      @item.apply(select: mailbox, conditions: ["NOT", "DELETED"], delete_cache: true, timeout: timeout)
    ensure
      @applied += @item.applied
    end
  end
end
