# encoding: utf-8
class Gw::Admin::Webmail::MailsController < Gw::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  include Gw::Controller::Admin::Mobile::Mail
  include RumiHelper
  include Gw::MailHelper
  include ApplicationHelper
  include ERB::Util
  helper Gw::MailHelper
  before_action :set_title
  layout :select_layout
  require 'base64'
  require 'zlib'

  rescue_from ActionController::InvalidAuthenticityToken, :with => :invalidtoken
  protect_from_forgery except: [:gw_forward]

  def logging_action
    remote_ip = request.env["HTTP_X_FORWARDED_FOR"] || request.remote_ip
    params[:url] = request.path_info
    feature_id =  params[:mailbox].blank? || params[:mailbox] != 'INBOX' ? nil : 'mail'
    access_log = System::AccessLog.new(user_id: Core.user.try(:id), user_code: Core.user.try(:account),
      user_name: Core.user.try(:name), ipaddress: remote_ip, controller_name: controller_name,
      action_name: action_name, parameters: params, feature_id: feature_id)
    access_log.save if access_log.logging?(params)
    return true
  end

  def pre_dispatch
    if params[:opt_id]
      Core.current_user = Core.user
      session.delete(:current_account)
    end
    if params[:action] == 'status' || params[:action] == 'mail_check'
      return
    end
    return redirect_to :action => 'index' if params[:reset]

    if params[:src] == 'mailto'
      mailto = parse_mailto(params[:uri])
      return redirect_to new_gw_webmail_mail_path(mailto.merge(:mailbox => 'INBOX'))
    end

    @limit = 20
    @new_window = params[:new_window].blank? ? nil : 1
    @mail_form_size = Gw::WebmailSetting.user_config_value(:mail_form_size, 'medium') unless params[:action] == 'index'
    @mailbox = Gw::WebmailMailbox.load_mailbox(params[:mailbox] || 'INBOX')
    @filter = ["UNDELETED"]
    @sort = get_sort_params(@mailbox.name)
    # アドレス帳検索 検索結果最大表示件数を取得
    @search_limit = Enisys.config.application['webmail.search_address_max_count'].to_i
  rescue => e
    rescue_mail(e)
  end

  def load_mailboxes
    load_starred_mails
    reload = flash[:gw_webmail_load_mailboxes]
    flash.delete(:gw_webmail_load_mailboxes)
    Gw::WebmailMailbox.load_quota(reload)
    Gw::WebmailMailbox.load_mailboxes(reload)
  end

  def reset_mailboxes(mailboxes = [:all])
    flash[:gw_webmail_load_mailboxes] = mailboxes.uniq
  end

  def load_starred_mails
    Util::Database.lock_by_name(Core.current_user.account) do
      cond = {:user_id => Core.current_user.id, :name => 'load_starred_mails'}
      setting = Gw::WebmailSetting.where(cond).first
      if setting
        mailbox_uids = JSON.parse(setting.value) rescue {}
        Gw::WebmailSetting.where(cond).delete_all
        Gw::WebmailMailbox.load_starred_mails(mailbox_uids)
      end
    end
  end

  def reset_starred_mails(mailbox_uids = {'INBOX' => :all})
    Util::Database.lock_by_name(Core.current_user.account) do
      cond = {:user_id => Core.current_user.id, :name => 'load_starred_mails'}
      setting = Gw::WebmailSetting.where(cond).first || Gw::WebmailSetting.new(cond)
      hash = JSON.parse(setting.value) rescue {}
      mailbox_uids.each do |mailbox, uids|
        hash[mailbox] ||= []
        hash[mailbox] = (hash[mailbox] + uids).uniq
      end
      setting.value = hash.to_json
      setting.save
    end
  end

  def load_quota
    Gw::WebmailMailbox.load_quota
  end

  def index
    return empty if params[:do] == 'empty'
    
    logging_action
    @opt_mail_id = params[:opt_id]
    params.delete(:opt_id)

    confs = Gw::WebmailSetting.user_config_values(
      [:mails_per_page, :mail_form_size, :mail_list_from_address, :mail_list_subject, :mail_open_window, :mail_address_history, :mail_pop_request])
    @limit = confs[:mails_per_page].blank? ? 20 : confs[:mails_per_page].to_i if !request.mobile?
    @mail_form_size = confs[:mail_form_size].blank? ? 'medium' : confs[:mail_form_size]
    @mail_list_from_address = confs[:mail_list_from_address]
    @mail_list_subject = confs[:mail_list_subject]
    @mail_open_window = confs[:mail_open_window]
    @mail_address_history = confs[:mail_address_history].blank? ? 10 : confs[:mail_address_history].to_i
    @label_confs = Gw::WebmailSetting.load_label_confs
    @polling_interval = confs[:mail_pop_request].to_i

    ## apply filters
    last_uid, recent, error = Gw::WebmailFilter.apply_recents
    reset_mailboxes([:all]) if recent
    if @mailbox.name == "INBOX"
      @filter += ["UID", "1:#{last_uid}"] # slows a little
    end
    if error
      flash.now[:notice] ||= ""
      flash.now[:notice]  += "（" + I18n.t("rumi.filter.message.filter_timeout") + "）"
    end

    ## search
    filter = make_search_filter
    @s_params = make_search_params
    @mailboxes = load_mailboxes
    @quota = load_quota
    @addr_histories = load_address_histories(@mail_address_history) if @mail_address_history != 0
    @items = Gw::WebmailMail.find(:all, select: @mailbox.name,
                                  conditions: filter,
                                  sort: @sort, 
                                  page: params[:page], 
                                  limit: @limit)
    @attack_domain_flg = []
    @attack_extension_flg = []
    @items.each do |item|
      # 警告ドメインチェック
      @attack_domain_flg << item.uid if item.node.attack_mail_domain_flg

      # 警告拡張子チェック
      @attack_extension_flg << item.uid if item.node.attack_mail_extension_flg
    end
    @attack_mail_message = Sys::AttackMailMessage.first
    if @attack_mail_message.blank?
      @attack_mail_message = Sys::AttackMailMessage.new
      @attack_mail_message.id = 1
      @attack_mail_message.title = t("rumi.sys.attackmail.message.table.default_title")
      @attack_mail_message.common = t("rumi.sys.attackmail.message.table.default_common").split("\\n").join("\n")
      @attack_mail_message.domain = t("rumi.sys.attackmail.message.table.default_domain")
      @attack_mail_message.extension = t("rumi.sys.attackmail.message.table.default_extension")
      @attack_mail_message.information = t("rumi.sys.attackmail.message.table.default_information")
      @attack_mail_message.save
      @attack_mail_message = Sys::AttackMailMessage.first
    end

    if params[:sort_starred] == '1'
      @starred_items = Gw::WebmailMail.find(:all, select: @mailbox.name,
                                           conditions: filter + " FLAGGED",
                                           sort: @sort,
                                           page: params[:page],
                                           limit: @limit)
      if @starred_items.size < @limit
        @unstarred_items = Gw::WebmailMail.find(:all, select: @mailbox.name,
                                                conditions: filter + " UNFLAGGED",
                                                sort: @sort, 
                                                page: params[:page], 
                                                limit: @limit - @starred_items.size)
      else
        @unstarred_items = []
      end
    end
  end

  def show
    @item  = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return error_auth unless @item
    confs = Gw::WebmailSetting.user_config_values([:mail_attachment_view, :mail_address_history])
    @mail_attachment_view = confs[:mail_attachment_view]
    @mail_address_history = confs[:mail_address_history].blank? ? 10 : confs[:mail_address_history].to_i
    @label_confs = Gw::WebmailSetting.load_label_confs
    if params[:download] == "eml"
      filename = @item.subject + ".eml"
      filename = filename.gsub(/[\/\<\>\|:"\?\*\\]/, '_')
      filename = URI::escape(filename) if Enisys.ie?(request.env['HTTP_USER_AGENT'])
      msg = @item.rfc822 || @item.mail.to_s
      return send_data(msg, :filename => filename,
        :type => 'message/rfc822', :disposition => 'attachment')
    elsif params[:download] == 'all'
      return download_all_attachments
    elsif params[:download]
      return download_attachment(params[:download])
    elsif params[:header]
      msg = @item.rfc822 || @item.mail.to_s
      msg = msg.slice(0, msg.index("\r\n\r\n")) rescue msg
      return send_data(msg.gsub(/\r\n/m, "\n"),
        :type => 'text/plain; charset=utf-8', :disposition => 'inline')
    elsif params[:source]
      msg = @item.rfc822 || @item.mail.to_s
      return send_data(msg.gsub(/\r\n/m, "\n"),
        :type => 'text/plain; charset=utf-8', :disposition => 'inline')
    elsif params[:show_html_image]
      return http_error(404) unless @item.html_mail?
      return respond_to do |format|
        format.xml { render :action => 'show_html' }
      end
    elsif params[:show_thumbnail_image]
      return render :action => 'show_thumbnail'
    end
    Core.title = '' if Core.title.blank?
    Core.title += " - #{@item.subject} - #{Core.current_user.email}"

    @all_webmail_setting = Sys::AllWebmailSetting.first
    if @item.html_mail?
      begin
        if @all_webmail_setting.htmlmail
          @html_mail_view = Gw::WebmailSetting.user_config_value(:html_mail_view, 'html')
        else
          @html_mail_view = 'text'
        end
      rescue
        @html_mail_view = Gw::WebmailSetting.user_config_value(:html_mail_view, 'html')
      end
    end
    @body_url = @all_webmail_setting.body_url rescue false

    attack_mail_domains = Sys::AttackMailDomain.select(:domain).map(&:domain)
    attack_mail_extensions = Sys::AttackMailExtension.select(:extension).map(&:extension)

    @attack_domain_flg = []
    @attack_extension_flg = []

    if @item.node.mailbox != "Sent"
      # 警告ドメインチェック
      @attack_domain_flg << @item.uid if @item.node.attack_mail_domain_flg

      # 警告拡張子チェック
      @attack_extension_flg << @item.uid if @item.node.attack_mail_extension_flg

      @attack_domain = ""
      msgs = @item.rfc822 || @item.mail.to_s
      if msgs.index("Return-Path")
        msgs = msgs.slice(0, msg.index("\r\n\r\n")) rescue msgs
        msg = msgs.gsub(/\r\n/m, "\n").split("\n")
        domain = ""
        msg.each do |m|
          if m.index("Return-Path") != nil
            address = m.split("@")
            domain = address[address.size-1].to_s.gsub(/(<|>)/, "")
            @attack_domain += "(" + domain + ")"
          end
        end
      else
        if @item.friendly_from_addr.present?
          domain = @item.friendly_from_addr.split("@")
          if domain[domain.size-1].present?
            d = domain[domain.size-1].to_s.gsub(/(<|>)/, "")
            @attack_domain += "(" + d + ")"
          end
        end
      end

      @attack_extension = ""
      if @item.has_attachments?
          @item.attachments.map(&:name).each do |at|
            filename = at.split(".")
            if filename.present?
              extension = filename[filename.size-1].to_s.gsub(/(<|>)/, "")
              if attack_mail_extensions.include?(extension)
                @attack_extension += "(" + extension + ")"
              end
            end
          end
        end

      @attack_mail_message = Sys::AttackMailMessage.first
      if @attack_mail_message.blank?
        @attack_mail_message = Sys::AttackMailMessage.new
        @attack_mail_message.id = 1
        @attack_mail_message.title = t("rumi.sys.attackmail.message.table.default_title")
        @attack_mail_message.common = t("rumi.sys.attackmail.message.table.default_common")
        @attack_mail_message.domain = t("rumi.sys.attackmail.message.table.default_domain")
        @attack_mail_message.extension = t("rumi.sys.attackmail.message.table.default_extension")
        @attack_mail_message.information = t("rumi.sys.attackmail.message.table.default_information")
        @attack_mail_message.save
        @attack_mail_message = Sys::AttackMailMessage.first
      end
    end

    if from = @item.parse_address(@item.friendly_from_addr).first
      @from_addr = CGI.escapeHTML(from.address)
      @from_name = ::NKF::nkf('-wx --cp932', from.name).gsub(/\0/, "") if from.name rescue nil
      @from_name = CGI.escapeHTML(@from_name || @from_addr)
    end rescue nil

    if @item.draft? && @item.mail.header[:bcc].blank? && @item.node
      @item.mail.header[:bcc] = @item.node.bcc
    end

    if @item.unseen?
      changed_mailboxes = []
      mailbox_uids = get_mailbox_uids(@mailbox, @item.uid)
      mailbox_uids.each do |mailbox, uids|
        num = Gw::WebmailMail.seen_all(mailbox, uids)
        if num > 0
          changed_mailboxes << mailbox
        end
      end
      reset_mailboxes(changed_mailboxes)

      @seen_flagged = true

      if @item.has_disposition_notification_to? && !@item.notified? && !@mailbox.draft_box?(:all) && !@mailbox.sent_box?(:all)
        @mdnRequest = mdn_request_mode
        if @mdnRequest && @mdnRequest == :auto
          begin
            send_mdn_message(@mdnRequest)
          rescue => e
            flash.now[:notice] = I18n.t("helpers.message.fail_auto_send")
          end
        end
      end
    end

    @mailboxes  = load_mailboxes
    filter = make_search_filter
    @s_params = make_search_params

    cond = { select: @mailbox.name, 
             conditions: filter,
             sort: @sort,
             sort_starred: params[:sort_starred]
           }
    @pagination = @item.single_pagination(params[:id], cond)

    @addr_histories = load_address_histories(@mail_address_history) if @mail_address_history != 0

    if request.xhr?
      html = render_to_string(template: 'gw/admin/webmail/mails/show.html.erb', layout: false)
      if Core.current_user.id == Core.user.id
        noread_cnt = @mailboxes.inject(0) do |cnt, box|
          unless box.name =~ /^(Sent|Drafts|Trash|Star)(\.|$)/
            cnt += Gw::WebmailMail.find(:all,
                                        select: box.name,
                                        conditions: "UNSEEN").size
          else
            cnt
          end
        end
      else
        noread_cnt = nil
      end

      return render json: {
        id: params[:id],
        html: html,
        tree: create_mail_tree(@mailbox, @mailboxes),
        mail_move_action: move_gw_webmail_mails_path(@mailbox.name),
        folder_move_action: move_folder_gw_webmail_mails_path(@mailbox.name),
        noread: noread_cnt
      }
    end
    _show @item
  end

  def download_all_attachments
    zipdata = ""
    Zip::Archive.open_buffer(zipdata, Zip::CREATE, Zip::DEFAULT_COMPRESSION) do |ar|
      filenames = @item.attachments.map {|at| at.name}
      filenames = unique_filenames(filenames)
      @item.attachments.each_with_index do |at, i|
        begin
          filename = filenames[i]
          filename = filename.gsub(/[\/\<\>\|:;"\?\*\\]/, '_')
          filename = filename.encode(Encoding::Windows_31J, :invalid => :replace, :undef => :replace, :replace => '_') if request.user_agent =~ /Windows/
          ar.add_buffer(filename, at.body)
        rescue Zip::Error => e
          # e
        end
      end
    end

    subject = @item.subject.gsub(/[\/\<\>\|:;"\?\*\\\r\n]/, '_')
    subject = subject.match(/^.{100}/).to_s if subject.length > 100
    subject = URI::escape(subject) if Enisys.ie?(request.user_agent)
    filename = sprintf("%07d_%s.zip", @item.uid, subject)
    send_data(zipdata, :type => 'application/zip', :filename => filename, :disposition => 'attachment')
  end

  def download_attachment(no)
    return http_error(404) unless no =~ /^[0-9]+$/
    return http_error(404) unless @file = @item.attachments[no.to_i]
    filedata     = @file.body
    content_type = @file.content_type
    disposition  = params[:disposition] ? params[:disposition] : (@file.image? ? 'inline' : 'attachment')
    if !params[:thumbnail].blank? && data = @file.thumbnail(width: params[:width] || 64, height: params[:height] || 64, format: :JPEG, quality: 70)
      filedata = data
      content_type = 'image/jpeg'
    end

    filename = convert_to_download_filename(@file.name)

    send_data(filedata, type: content_type, filename: filename, disposition: disposition)
  end

  def new
    return false if no_email?
    @form_action = "create"

    @item = Gw::WebmailMail.new
    @item.tmp_id  = Sys::File.new_tmp_id
    if default_template
      @item.in_to      = default_template.to
      @item.in_cc      = default_template.cc
      @item.in_bcc     = default_template.bcc
      @item.in_subject = default_template.subject
      @item.in_body    = default_template.body
    end
    if default_sign_body
      @item.in_body  ||= ""
      @item.in_body   += "\n\n#{default_sign_body}"
    end

    load_address_from_flash

    @item.in_to      = NKF::nkf('-w', params[:to]) if params[:to]
    @item.in_cc      = NKF::nkf('-w', params[:cc]) if params[:cc]
    @item.in_bcc     = NKF::nkf('-w', params[:bcc]) if params[:bcc]
    @item.in_subject = NKF::nkf('-w', params[:subject]) if params[:subject]
    @item.in_body    = "#{NKF::nkf('-w', params[:body])}\n\n#{@item.in_body}" if params[:body]

    @item.in_format = Gw::WebmailMail::FORMAT_TEXT
  end

  def edit
    return false if no_email?
    @form_action = "update"
    @form_method = "put"
    @ref = Gw::WebmailMail.find_by_uid(params[:id],
                                      select: @mailbox.name,
                                      conditions: @filter)
    return http_error(404) unless @ref
    ref_node = @ref.find_node

    @item = Gw::WebmailMail.new(params[:item])
    @item.tmp_id     = Sys::File.new_tmp_id
    @item.in_to      = @ref.friendly_to_addrs.join(',')
    @item.in_cc      = @ref.friendly_cc_addrs.join(',')
    @item.in_bcc     = @ref.friendly_bcc_addrs.join(',')
    @item.in_bcc     = ref_node.bcc if @item.in_bcc.blank? && ref_node
    @item.in_subject = @ref.subject
    if params[:mail_view] == Gw::WebmailMail::FORMAT_HTML && @ref.html_mail?
      @item.in_html_body = @ref.html_body_for_edit
      @item.in_format    = Gw::WebmailMail::FORMAT_HTML
    else
      @item.in_body      = @ref.text_body
      @item.in_format    = Gw::WebmailMail::FORMAT_TEXT
    end
    @item.request_mdn = 1 if @ref.has_disposition_notification_to?

    load_address_from_flash

    load_attachments(@ref, @item)
    render action: :new
  end

  def answer
    return false if no_email?
    @form_action = "answer"
    @ref = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return http_error(404) unless @ref
    @item = Gw::WebmailMail.new(params[:item])
    @item.reference = @ref
    if request.post?
      return send_message(@item, @ref) do
        mailbox_uids = get_mailbox_uids(@mailbox, @ref.uid)
        mailbox_uids.each do |mailbox, uids|
          Core.imap.select(mailbox)
          Core.imap.uid_store(uids, "+FLAGS", [:Answered])
        end
      end
    end
    @item.tmp_id     = Sys::File.new_tmp_id
    @item.in_to      = @ref.friendly_reply_to_addrs(!params[:all].blank?).join(', ')
    @item.in_subject = "Re: " + @ref.subject
    
    if params[:mail_view] == Gw::WebmailMail::FORMAT_HTML && @ref.html_mail?
      quot_body = "<p style=\"margin:0px; padding:0px;\"></p>#{@ref.referenced_html_body}" if params[:qt]
      sign_body = Util::String.text_to_html("\n" + default_sign_body) if default_sign_body
      @item.in_html_body = concat_mail_body(quot_body, sign_body)
      @item.in_format = Gw::WebmailMail::FORMAT_HTML
    else
      quot_body = "\n\n#{@ref.referenced_body}" if params[:qt]
      sign_body = "\n\n#{default_sign_body}" if default_sign_body
      @item.in_body = concat_mail_body(quot_body, sign_body)
      @item.in_format = Gw::WebmailMail::FORMAT_TEXT
    end

    if params[:all]
      @item.in_cc = @ref.friendly_cc_addrs.join(', ')
    end
    load_address_from_flash
    render action: :new
  end

  def forward
    return false if no_email?
    @form_action = "forward"
    @ref = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return http_error(404) unless @ref
    @item = Gw::WebmailMail.new(params[:item])

    if request.post?
      return send_message(@item, @ref) do
        mailbox_uids = get_mailbox_uids(@mailbox, @ref.uid)
        mailbox_uids.each do |mailbox, uids|
          Core.imap.select(mailbox)
          Core.imap.uid_store(uids, "+FLAGS", "$Forwarded")
        end
      end
    end
    @item.tmp_id      = Sys::File.new_tmp_id
    @item.in_subject  = "Fw: " + @ref.subject

    if params[:mail_view] == Gw::WebmailMail::FORMAT_HTML && @ref.html_mail?
      quot_body = "<p style=\"margin:0px; padding:0px;\"></p>#{@ref.referenced_html_body(:forward)}"
      sign_body = Util::String.text_to_html("\n" + default_sign_body) if default_sign_body
      @item.in_html_body = concat_mail_body(quot_body, sign_body)
      @item.in_format = Gw::WebmailMail::FORMAT_HTML
    else
      quot_body = "\n\n#{@ref.referenced_body(:forward)}"
      sign_body = "\n\n#{default_sign_body}" if default_sign_body
      @item.in_body = concat_mail_body(quot_body, sign_body)
      @item.in_format = Gw::WebmailMail::FORMAT_TEXT
    end
    load_address_from_flash
    load_attachments(@ref, @item)
    render(action: :new)
  end

  def gwcircular_forward
    return false if no_email?
    @form_action = "forward"
    @ref = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return http_error(404) unless @ref
    @item = @ref
    _url = Enisys::Config.application["gw.root_url"]
    @forward_form_url = URI.join(_url, "/gwcircular/forward").to_s
    @target_name = "gwcircular_form"
    forward_setting
  end

  def gwbbs_forward
    return false if no_email?
    @form_action = "forward"
    @ref = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return http_error(404) unless @ref
    @item = @ref
    _url = Enisys::Config.application["gw.root_url"]
    @forward_form_url = URI.join(_url, "/gwbbs/forward_select").to_s
    @target_name = "gwbbs_form_select"
    forward_setting
  end

  def user_agent_info(user_agent)
    [ /(MSIE) (\d+)\.(\d*)/,
      /(Firefox)\/(\d+)\.(\d*)/,
      /(?=.*(Opera)[\s|\/])(?=.*Version\/(\d+)\.(\d*))/,
      /(Chrome)\/(\d+)\.(\d*)/,
      /(?=.*(Safari)\/)(?=.*Version\/(\d+)\.(\d*))/,
      Enisys.ie11_regexp
    ].each do |regexp|
      if user_agent =~ regexp
        return $1, $2.to_i, $3.to_i
      end
    end
    return "unknown"
  end

  def data_uri_scheme_limit_size
    limit_size = 1024**3
    agent, version, subversion = user_agent_info(request.user_agent)

    case agent
    when 'MSIE', 'Trident'
      if version < 8
        limit_size = 0
      elsif version == 8
        limit_size = 32*1024
      end
    when 'Firefox'
      if version < 2
        limit_size = 0
      elsif version == 2
        limit_size = 100*1024
      end
    when 'Opera'
      if version < 7
        limit_size = 0
      elsif version == 7 && subversion < 20
        limit_size = 4*1024
      end
    when 'Chrome', 'Safari'
    else
      limit_size = 0
    end
    return limit_size
  end

  def forward_setting
    #機能間転送の為の処理
    #本文の処理
    @mail_text_body = "-------- Original Message --------<br />"
    #本文_件名
    _title = @item.subject.present? ? @item.subject : ""
    @mail_text_body << I18n.t("rumi.forward_message.title") + _title + "<br />"
    #本文_作成日時
    _date = @item.date.present? ? I18n.l(@item.date.to_time) : ""
    @mail_text_body << I18n.t("rumi.forward_message.date") + _date + "<br />"
    #本文_差出人アドレス
    _friendly_from_addr = @item.friendly_from_addr.present? ? @item.friendly_from_addr : ""
    @mail_text_body << I18n.t("rumi.forward_message.from") + _friendly_from_addr + "<br />"
    #本文_宛先アドレス及びCCのアドレス
    _friendly_to_addrs = @item.friendly_to_addrs.present? ? @item.friendly_to_addrs.join('，') : ""
    @mail_text_body << I18n.t("rumi.forward_message.to") + _friendly_to_addrs + "<br />"
    if @item.friendly_cc_addrs.present?
      _friendly_cc_addrs = @item.friendly_cc_addrs.present? ? @item.friendly_cc_addrs.join('，') : ""
      @mail_text_body << "CC: " + _friendly_cc_addrs + "<br />"
    end
    #本文_メール本文
    @mail_text_body << "----<br />"

    if params[:show_html_image_on]
      limit_size = data_uri_scheme_limit_size
      @item.inline_contents(:show_image => true, :embed_image => limit_size > 0, :embed_image_size_limit => limit_size)
    end
    @item.inline_contents.each_with_index do |inline, idx|
      if params[:mail_view] == "text"
        all_webmail_setting = Sys::AllWebmailSetting.first
        body_url = all_webmail_setting.body_url rescue false
        @mail_text_body << mail_text_wrap(inline.text_body, 10, :auto_link => false)
      else
        if inline.content_type == "text/plain" || inline.html_body.blank?
          body = inline.text_body.split("\n")
          body.each do |b|
            @mail_text_body << "#{h(b)}<br />"
          end
        else
          @mail_text_body << "#{inline.html_body}"
        end
      end
    end
    @mail_text_body << "------ Original Message Ends ------<br />"
    @mail_text_body = Base64.encode64(@mail_text_body).split().join()

    @tmp = ""
    @name = ""
    @content_type = ""
    @size = ""
    @item.attachments.each do |at|
      @tmp.concat "," if @tmp.present?
      tmp = Zlib::Deflate.deflate(at.body, Zlib::BEST_COMPRESSION)
      @tmp.concat Base64.encode64(tmp).split().join()
      @name.concat "," if @name.present?
      @name.concat at.name.to_s
      @content_type.concat "," if @content_type.present?
      @content_type.concat at.content_type.to_s
      @size.concat "," if @size.present?
      @size.concat at.size.to_s
    end
  end

  def gw_forward
    return false if no_email?
    @form_action = "create"
    @item = Gw::WebmailMail.new
    @item.tmp_id  = Sys::File.new_tmp_id
    load_address_from_flash
    @item.in_to      = NKF::nkf('-w', params[:to]) if params[:to]
    @item.in_cc      = NKF::nkf('-w', params[:cc]) if params[:cc]
    @item.in_bcc     = NKF::nkf('-w', params[:bcc]) if params[:bcc]
    @item.in_subject = NKF::nkf('-w', params[:subject]) if params[:subject]
    if params[:body]
      __body = Base64.decode64(params[:body]).to_s
      if __body.include?("<")
        @item.in_html_body = __body.force_encoding("utf-8")
        @item.in_html_body << "\n\n#{@item.in_body}"
        @item.in_format    = Gw::WebmailMail::FORMAT_HTML
      else
        @item.in_body = "#{NKF::nkf('-w', __body.force_encoding("utf-8"))}\n\n#{@item.in_body}"
        @item.in_format    = Gw::WebmailMail::FORMAT_TEXT
      end
    end

    if params[:tmp].present?
      forward = params[:tmp].split(",")
      name = params[:name].split(",")
      size = params[:size].split(",")
      cnt = 0
      forward.each do |f|
        total_size = size[cnt].to_i
        cond = { tmp_id: @item.tmp_id.to_s }
        Gw::WebmailMailAttachment.where(cond).each {|c| total_size += c.size.to_i }
        total_size_limit = Enisys::Config.application["webmail.attachment_file_max_size"].to_s + "MB" || "5 MB"
        limit_value = total_size_limit.gsub(/(.*?)[^0-9].*/, '\\1').to_i * (1024**2)
        mb = size[cnt].to_f / 1.megabyte.to_f
        mb = (mb * 100).to_i
        mb = sprintf('%g', mb.to_f / 100)
        if total_size > limit_value
          @error_messeage = I18n.t("helpers.message.fail_temp_size", total_size_limit: total_size_limit, mb: mb)
          next
        end

        file = Gw::WebmailMailAttachment.new(cond)

        begin
          at = Base64.decode64(f)
          tmpfile = Sys::Lib::File::Tempfile.new({
            :data     => Zlib::Inflate.inflate(at),
            :filename => name[cnt]
          })
          rs = file.save_file(tmpfile)
        rescue => e
          flash[:notice] = I18n.t("helpers.message.fail_file_regist") + "#{e}"
          next
        end
        unless rs
          raise file.errors.full_messages.join("\n")
          next
        end

        unless FileTest.file?(file.upload_path)
          flash[:notice] = I18n.t("helpers.message.no_exist_file")+ "(#{file.upload_path})"
          next
        end
        ## garbage collect
        Sys::File.garbage_collect if rand(100) == 0
        cnt = cnt + 1
      end
    end
    render(action: :new)
  end

  def resend
    return false if no_email?

    @form_action = "create"

    @ref = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return http_error(404) unless @ref
    @item = Gw::WebmailMail.new
    @item.tmp_id     = Sys::File.new_tmp_id
    @item.in_to      = @ref.friendly_to_addrs.join(', ')
    @item.in_cc      = @ref.friendly_cc_addrs.join(', ')
    @item.in_bcc     = @ref.friendly_bcc_addrs.join(', ')
    @item.in_subject = @ref.subject
    if params[:mail_view] == Gw::WebmailMail::FORMAT_HTML && @ref.html_mail?
      @item.in_html_body = @ref.html_body
      @item.in_format = Gw::WebmailMail::FORMAT_HTML
    else
      @item.in_body = @ref.text_body
      @item.in_format = Gw::WebmailMail::FORMAT_TEXT
    end
    @item.request_mdn = 1 if @ref.has_disposition_notification_to?
    load_address_from_flash
    load_attachments(@ref, @item)
    render(action: :new)
  end

  def create
    return false if no_email?
    @form_action = "create"
    if params[:id] && params[:id] != '0'
      @ref = Gw::WebmailMail.find_by_uid(params[:id], :select => @mailbox.name, :conditions => @filter)
      return http_error(404) unless @ref
    end
    @item = Gw::WebmailMail.new(params[:item])
    send_message(@item, @ref) do
      if !params[:remain_draft] && @ref && (@mailbox.draft_box?(:all) || @mailbox.star_box?)
        mailbox_uids = get_mailbox_uids(@mailbox, @ref.uid)
        mailbox_uids.each do |mailbox, uids|
          num = Gw::WebmailMail.delete_all(mailbox, uids, true)
          if num > 0
            Gw::WebmailMailNode.delete_nodes(mailbox, uids)
          end
        end
      end
    end
  end

  def update
    return false if no_email?
    @form_action = "update"
    @form_method = "put"

    @ref = Gw::WebmailMail.find_by(params[:id], :select => @mailbox.name, :conditions => @filter)
    return http_error(404) unless @ref

    @item = Gw::WebmailMail.new(params[:item])
    send_message(@item, @ref) do
      Gw::WebmailMailNode.delete_nodes(@mailbox.name, @ref.uid)
      @ref.destroy(true)
    end
  end

  def send_message(item, ref, &block)
    config = Gw::WebmailSetting.user_config_value(:mail_from)
    ma = Mail::Address.new
    ma.address      = Core.current_user.email
    ma.display_name = Core.current_user.name if config != "only_address"
    item.in_from = ma.to_s

    ## submit/file
    if !params[:commit_file].blank?
      item.valid?(:file)
      if request.xhr?
        return
      else
        return render(:action => :new)
      end
    end

    ## submit/destroy
    if !params[:commit_destroy].blank?
      item.delete_tmp_attachments
      return redirect_to(:action => :close)
    end


    ## submit/draft
    if !params[:commit_draft].blank?
      unless item.valid?(:draft)
        #@mailboxes  = load_mailboxes
        return render(:action => :new)
      end
      return save_as_draft(item, ref, &block)
    end

    ## submit/send
    unless item.valid?(:send)
      #@mailboxes  = load_mailboxes
      return render(:action => :new)
    end

    begin
      mail = item.prepare_mail(request)
      mail.delivery_method(:smtp, ActionMailer::Base.smtp_settings)
      sent = mail.deliver
      item.delete_tmp_attachments
      item.save_address_history
    rescue => e
      flash.now[:error] = I18n.t("rumi.mails.view.message.errors.cannot_send") + "（#{e}）"
      respond_to do |format|
        format.html { render :action => :new }
        format.xml  { render :xml => item.errors, :status => :unprocessable_entity }
      end
      return
    end

    yield if block_given?

    ## save to 'Sent'
    begin
      imap = Core.imap
      imap.create("Sent") unless imap.list("", "Sent") rescue nil
      item.mail = sent
      timeout(60) { imap.append("Sent", item.for_save.to_s, [:Seen], Time.now) }
    rescue => e
      flash[:error] = I18n.t("helpers.message.fail_tray_regist") + "（#{e}）"
    end

    status         = params[:_created_status] || :created
    location       = url_for(:action => :close)
    respond_to do |format|
      format.html { redirect_to(location) }
      format.xml  { render :xml => item.to_xml(:dasherize => false), :status => status, :location => location }
    end
  end

  def save_as_draft(item, ref, &block)
    begin
      mail = item.prepare_mail(request)
      imap = Core.imap
      imap.create("Drafts") unless imap.list("", "Drafts") rescue nil
      item.mail = mail
      flags = [:Seen, :Draft]
      flags << :Flagged if ref && ref.starred?
      timeout(30) { imap.append("Drafts", item.for_save.to_s, flags, Time.now) }
      item.delete_tmp_attachments

      yield if block_given?
    rescue => error
      item.errors.add :base, I18n.t("helpers.message.fail_draft") + "（#{error}）"
      flash.now[:notice] = I18n.t("helpers.message.fail_draft") + "（#{error}）"
      respond_to do |format|
        format.html { render :action => :new }
        format.xml  { render :xml => item.errors, :status => :unprocessable_entity }
      end
      return
    end
    status         = params[:_created_status] || :created
    location       = url_for(:action => :close)
    respond_to do |format|
      format.html { redirect_to(location) }
      format.xml  { render :xml => item.to_xml(:dasherize => false), :status => status, :location => location }
    end
  end

  def destroy
    @item  = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return error_auth unless @item

    changed_num = 0
    changed_mailbox_uids = {}

    mailbox_uids = get_mailbox_uids(@mailbox, @item.uid)
    mailbox_uids.each do |mailbox, uids|
      num = Gw::WebmailMail.delete_all(mailbox, uids, params[:real])
      if num > 0
        Gw::WebmailMailNode.delete_nodes(mailbox, uids)
        changed_mailbox_uids['Trash'] = [:all]
      end
      changed_num += num if mailbox !~ /^(Star)$/
    end

    if changed_num > 0
      reset_mailboxes([:all])
      reset_starred_mails(changed_mailbox_uids) if @item.starred?

      flash[:notice] = I18n.t("helpers.message.move_trash")
      flash[:notice] = I18n.t("helpers.message.delete_mail") unless @new_window if params[:real]
      respond_to do |format|
        format.html do
          action = @new_window ? :close : :index
          redirect_to url_for(:action => action)
        end
        format.xml  { head :ok }
      end
    else
      flash[:error] = I18n.t("helpers.message.fail_delete_mail")
      respond_to do |format|
        format.html do
          action = @new_window ? :close : :index
          redirect_to url_for(:action => action)
        end
        format.xml  { render xml: @item.errors, status: :unprocessable_entity }
      end
    end
  end

  ## move_all or move_one
  def move
    if !params[:item] || !params[:item][:ids]
      return redirect_to(url_for(:action => :index, :page => params[:page]))
    end

    uids = params[:item][:ids].collect{|k, v| k.to_s =~ /^[0-9]+$/ ? k.to_i : nil }
    cond = ["UID", uids] + @filter

    if !params[:item][:mailbox]
      @items = Gw::WebmailMail.find(:all, select: @mailbox.name, conditions: cond)
      @mailboxes = load_mailboxes

      confs = Gw::WebmailSetting.user_config_values([:mail_address_history])
      @mail_address_history = confs[:mail_address_history].blank? ? 10 : confs[:mail_address_history].to_i
      @addr_histories = load_address_histories(@mail_address_history) if @mail_address_history != 0

      return render :template => 'gw/admin/webmail/mails/move'
    end

    changed_num = 0
    changed_mailbox_uids = {}
    include_starred_uid = Gw::WebmailMail.include_starred_uid?(@mailbox.name, uids)

    mailbox_uids = get_mailbox_uids(@mailbox, uids)
    mailbox_uids.each do |mailbox, uids|
      if mailbox =~ /^(Star)$/
        num = 0
        if params[:copy].blank? #move
          num = Gw::WebmailMail.delete_all(mailbox, uids, true)
          Gw::WebmailMailNode.delete_nodes(mailbox, uids) if num > 0
        end
      else
        if params[:copy].blank? #move
          num = Gw::WebmailMail.move_all(mailbox, params[:item][:mailbox], uids)
          Gw::WebmailMailNode.delete_nodes(mailbox, uids) if num > 0
        else
          num = Gw::WebmailMail.copy_all(mailbox, params[:item][:mailbox], uids)
        end
        changed_num += num
      end
      if num > 0
        changed_mailbox_uids[params[:item][:mailbox]] = [:all]
      end
    end

    reset_mailboxes([:all])
    reset_starred_mails(changed_mailbox_uids) if include_starred_uid

    label = params[:copy].blank? ? I18n.t("rumi.mails.view.menu.move") : I18n.t("rumi.mails.view.menu.copy")
    flash[:notice] = I18n.t("helpers.message.ope_mail", changed_num: changed_num, label: label).force_encoding('utf-8') unless @new_window
    action = @new_window ? :close : :index
    redirect_to url_for(:action => action, :page => params[:page])
  end

  ## move_folder
  def move_folder
    srcbox = Gw::WebmailMailbox.load_mailbox(params[:item][:srcbox])
    distbox = Gw::WebmailMailbox.load_mailbox(params[:item][:distbox])
    new_name = "#{distbox.name}.#{Net::IMAP.encode_utf7(srcbox.title)}"
    url_param = {action: :index, page: params[:page]}
    begin
      ActiveRecord::Base.transaction do
        Gw::WebmailMailNode.delete_nodes(srcbox.name)
        Core.imap.rename(srcbox.name, new_name)

        uids = Gw::WebmailMailNode.find_ref_nodes(srcbox.name).map{|x| x.uid }
        Core.imap.select('Star')
        num = Core.imap.uid_store(uids, "+FLAGS", [:Deleted]).size rescue 0
        Core.imap.expunge
        if num > 0
          Gw::WebmailMailNode.delete_ref_nodes(srcbox.name)
          mailbox_uids = flash[:gw_webmail_load_starred_mails]
          flash.delete(:gw_webmail_load_starred_mails)
          Gw::WebmailMailbox.load_starred_mails(mailbox_uids)
        end

        flash[:gw_webmail_load_mailboxes] = [:all]
      end
      url_param[:mailbox] = new_name if params[:mailbox] == params[:item][:srcbox]
      flash[:notice] = I18n.t('rumi.attachment.message.folder_moved')
    rescue => e
      case e.message
      when /Mailbox already exists:/
        msg = I18n.t('rumi.attachment.message.already_exists', file: new_name)
      when /Can't rename mailbox under its own child./
        msg = I18n.t('rumi.attachment.message.reverse_folder')
      else
        msg = e.message
      end
      flash[:notice] = msg
    end

    redirect_to url_for(url_param)
  end

  ## destroy_all
  def delete
    if !params[:item] || !params[:item][:ids]
      return redirect_to url_for(:action => :index, :page => params[:page])
    end

    uids = params[:item][:ids].collect{|k, v| k.to_s =~ /^[0-9]+$/ ? k.to_i : nil }

    changed_num = 0
    changed_mailbox_uids = {}
    include_starred_uid = Gw::WebmailMail.include_starred_uid?(@mailbox.name, uids)

    mailbox_uids = get_mailbox_uids(@mailbox, uids)
    mailbox_uids.each do |mailbox, uids|
      num = Gw::WebmailMail.delete_all(mailbox, uids, params[:real])
      if num > 0
        Gw::WebmailMailNode.delete_nodes(mailbox, uids)
        changed_mailbox_uids['Trash'] = [:all]
      end
      changed_num += num if mailbox !~ /^(Star)$/
    end

    reset_mailboxes([:all])
    reset_starred_mails(changed_mailbox_uids) if include_starred_uid
    flash[:notice] = I18n.t("helpers.message.move_trash_mail", changed_num: changed_num).force_encoding('utf-8')
    flash[:notice] = I18n.t("helpers.message.delete_trash_mail", changed_num: changed_num).force_encoding('utf-8') if params[:real]
    redirect_to url_for(:action => :index, :page => 1)
  end

  def seen
    if !params[:item] || !params[:item][:ids]
      return redirect_to url_for(:action => :index, :page => params[:page])
    end

    uids = params[:item][:ids].collect{|k, v| k.to_s =~ /^[0-9]+$/ ? k.to_i : nil }

    changed_num = 0
    changed_mailboxes = []

    mailbox_uids = get_mailbox_uids(@mailbox, uids)
    mailbox_uids.each do |mailbox, uids|
      num = Gw::WebmailMail.seen_all(mailbox, uids)
      if num > 0
        changed_mailboxes << mailbox
        changed_mailboxes << 'Star'
      end
      changed_num += num if mailbox !~ /^(Star)$/
    end

    reset_mailboxes(changed_mailboxes)

    flash[:notice] = I18n.t("helpers.message.seen_mail", changed_num: changed_num).force_encoding('utf-8')
    redirect_to url_for(:action => :index, :page => params[:page])
  end

  def unseen
    if !params[:item] || !params[:item][:ids]
      return redirect_to url_for(:action => :index, :page => params[:page])
    end

    uids = params[:item][:ids].collect{|k, v| k.to_s =~ /^[0-9]+$/ ? k.to_i : nil }

    changed_num = 0
    changed_mailboxes = []

    mailbox_uids = get_mailbox_uids(@mailbox, uids)
    mailbox_uids.each do |mailbox, uids|
      num = Gw::WebmailMail.unseen_all(mailbox, uids)
      if num > 0
        changed_mailboxes << mailbox
      end
      changed_num += num if mailbox !~ /^(Star)$/
    end

    reset_mailboxes(changed_mailboxes)
    flash[:notice] = I18n.t("helpers.message.unread_mail", changed_num: changed_num).force_encoding('utf-8')
    redirect_to url_for(:action => :index, :page => params[:page])
  end

  def empty
    reset_mailboxes([:all])
    changed_mailbox_uids = {}

    mailboxes = load_mailboxes
    mailboxes.reverse.each do |box|
      if box.trash_box?(:children)
        begin
          Gw::WebmailMailNode.delete_nodes(box.name)
          Core.imap.delete(box.name)

          uids = Gw::WebmailMailNode.find_ref_nodes(box.name).map{|x| x.uid}
          num = Gw::WebmailMail.delete_all('Star', uids)
          if num > 0
            Gw::WebmailMailNode.delete_ref_nodes(box.name)
            changed_mailbox_uids[box.name] = [:all]
          end
        rescue => e
        end
      end
    end

    Core.imap.select(@mailbox.name)
    uids = Core.imap.uid_search(@filter, "utf-8")

    mailbox_uids = get_mailbox_uids(@mailbox, uids)
    mailbox_uids.each do |mailbox, uids|
      num = Gw::WebmailMail.delete_all(mailbox, uids, true)
      if num > 0
        Gw::WebmailMailNode.delete_nodes(mailbox, uids)
        changed_mailbox_uids[mailbox] = [:all]
      end
    end

    reset_mailboxes([:all])
    reset_starred_mails(changed_mailbox_uids)

    flash[:notice] = I18n.t("helpers.message.empty_trash")
    respond_to do |format|
      format.html { redirect_to url_for(:action => :index) }
      format.xml  { head :ok }
    end
  end

  def send_mdn
    @item = Gw::WebmailMail.find_by(params[:id], :select => @mailbox.name, :conditions => @filter)
    return error_auth unless @item && @item.has_disposition_notification_to?

    if params[:mobile]
      begin
        send_mdn_message(params[:send_mode])
        flash[:notice] = I18n.t("helpers.message.send_read_receipt")
      rescue => e
        flash[:notice] = I18n.t("helpers.message.fail_send_read_receipt")
      end
      return redirect_to url_for(:action => :show)
    else
      send_mdn_message(params[:send_mode])
    end
  end

  def reset_address_history
    Gw::WebmailMailAddressHistory.where(user_id: Core.current_user.id).delete_all
    flash[:notice] = I18n.t("helpers.message.reset_quick")
    redirect_to url_for(action: :index, page: params[:page])
  end

  def register_spam
    if !params[:item] || !params[:item][:ids]
      return redirect_to url_for(action: :index, page: params[:page])
    end

    uids = params[:item][:ids].collect{|k, v| k.to_s =~ /^[0-9]+$/ ? k.to_i : nil }
    cond = ["UID", uids] + @filter
    items = Gw::WebmailMail.find(:all, select: @mailbox.name,
                                       conditions: cond,
                                       sort: @sort)
    if items.count == 0
      return redirect_to url_for(action: :index, page: params[:page])
    end

    Gw::WebmailFilter.register_spams(items)

    changed_num = 0
    changed_mailbox_uids = {}
    include_starred_uid = Gw::WebmailMail.include_starred_uid?(@mailbox.name, uids)

    mailbox_uids = get_mailbox_uids(@mailbox, uids)
    mailbox_uids.each do |mailbox, uids|
      num = Gw::WebmailMail.delete_all(mailbox, uids)
      if num > 0
        Gw::WebmailMailNode.delete_nodes(mailbox, uids)
        changed_mailbox_uids['Trash'] = [:all]
      end
      changed_num += num if mailbox !~ /^(Star)$/
    end

    reset_mailboxes([:all])
    reset_starred_mails(changed_mailbox_uids) if include_starred_uid
    flash[:notice] = I18n.t("helpers.message.nuisance_regist", count: items.count)
    redirect_to url_for(:action => :index, :page => params[:page])
  end

  def star
    @item  = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return error_auth unless @item

    starred = @item.starred?
    changed_mailbox_uids = {}

    mailbox_uids = get_mailbox_uids(@mailbox, @item.uid)
    mailbox_uids.each do |mailbox, uids|
      if starred
        num = Gw::WebmailMail.unstar_all(mailbox, uids)
      else
        num = Gw::WebmailMail.star_all(mailbox, uids)
      end
      if num > 0
        changed_mailbox_uids[mailbox] = uids
      end
    end

    reset_mailboxes([:all])
    reset_starred_mails(changed_mailbox_uids)

    if request.mobile? && !request.smart_phone?
      s_params = make_search_params
      if params[:from] == 'list' || @mailbox.name =~ /^(Star)$/
        redirect_to url_for(s_params.merge(:action => :index, :id => params[:id], :mailbox => @mailbox.name, :mobile => :list))
      else
        redirect_to url_for(s_params.merge(:action => :show, :id => params[:id], :mailbox => @mailbox.name))
      end
    else
      render :text => "OK"
    end
  end

  def label
    @item  = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    return error_auth unless @item

    @label_confs = Gw::WebmailSetting.load_label_confs

    label = params[:label].to_i
    labeled = @item.labeled?(label)

    mailbox_uids = get_mailbox_uids(@mailbox, @item.uid)
    mailbox_uids.each do |mailbox, uids|
      if label == 0
        Gw::WebmailMail.unlabel_all(mailbox, uids)
      elsif labeled
        Gw::WebmailMail.unlabel_all(mailbox, uids, label)
      else
        Gw::WebmailMail.label_all(mailbox, uids, label)
      end
    end
    @item  = Gw::WebmailMail.find_by_uid(params[:id], select: @mailbox.name, conditions: @filter)
    render layout: false
  end

  def status
    imap_settings = Enisys.config.imap_settings
    smtp_settings = ActionMailer::Base.smtp_settings
    imap_sock = TCPSocket.open(imap_settings[:address], imap_settings[:port])
    smtp_sock = TCPSocket.open(smtp_settings[:address], smtp_settings[:port])
    if imap_sock && smtp_sock
      @status = "OK"
      #if form_authenticity_token != form_authenticity_param
      #  @status = "NG TokenError"
      #end
    end
  rescue => e
    @status = "NG"
  ensure
    imap_sock.close if imap_sock
    smtp_sock.close if smtp_sock
  end

  def dnd_upload
    params[:commit_file] = 'DnD'
    params[:item] = {
      in_files: params[:dnd_file].values,
      tmp_id: params[:dnd_tmp_id]
    }

    @item = Gw::WebmailMail.new(params[:item])
    send_message(@item, nil)
    if @item.errors.full_messages.present?
      raise @item.errors.full_messages.join("\n")
    end

    render json: { status: 'OK' }
  rescue => e
    if e.message.present?
      render json: { status: 'NG', message: e.message }
    else
      render json: { status: 'NG', message: I18n.t('rumi.attachment.message.failure_upload') }
    end
  end

  def mail_check
    conf = Enisys.config.imap_settings
    imap = Core.imap
    imap.select('inbox')
    msg = imap.search(['RECENT'])
    if msg.present?
      render json: { status: 'OK', new_mail: true }
    else
      render json: { status: 'OK', new_mail: false }
    end
  rescue => e
    Core.terminate
    render json: { status: 'NG' }
  end

  def attack_mail_change
    user = Sys::User.where(id: params[:user_id]).first
    if params[:current_account] != ""
      _user = Sys::User.where(code: params[:current_account]).first
      user = _user if _user.present?
    end
    item = Gw::WebmailMailNode.where(uid: params[:id], mailbox: params[:mailbox], user_id: user.id).first
    if item.present?
      item.attack_mail_domain_flg = false
      item.attack_mail_extension_flg = false
      item.save
      if item.ref_uid.present? && item.ref_mailbox.present?
        ref_item  = Gw::WebmailMailNode.where(uid: item.ref_uid.to_s, mailbox: item.ref_mailbox.to_s).first
        if ref_item.present?
          ref_item.attack_mail_domain_flg = false
          ref_item.attack_mail_extension_flg = false
          ref_item.save
        end
      else
        star_item = Gw::WebmailMailNode.where(ref_uid: item.uid.to_s, mailbox: "Star").first
        if star_item.present?
          star_item.attack_mail_domain_flg = false
          star_item.attack_mail_extension_flg = false
          star_item.save
        end
      end
    end
    render json: { status: 'OK' }
  end

protected

  def select_layout
    layout = "admin/gw/webmail"
    case params[:action].to_sym
    when :new, :edit, :answer, :forward, :gw_forward, :close, :create, :update, :resend
      layout = "admin/gw/mail_form"
    when :gwcircular_forward, :gwbbs_forward
      layout = "admin/gw/forward_form"
    when :show, :move
      unless params[:new_window].blank?
        layout = "admin/gw/mail_form"
      end
    end
    layout
  end

  def no_email?
    if Core.current_user.email.blank?
      error = I18n.t('helpers.message.not_regist_address')
      render(:text => error, :layout => true)
      return true
    end
    return nil
  end

  def default_sign_body
    return @default_sign.body if @default_sign
    if request.mobile?
      @default_sign = Gw::WebmailSign.new
    else
      @default_sign = (Gw::WebmailSign.default_sign || Gw::WebmailSign.new)
    end
    @default_sign.body
  end

  def default_template
    return @default_template if @default_template
    if request.mobile?
      @default_template = Gw::WebmailTemplate.new
    else
      @default_template = (Gw::WebmailTemplate.default_template || Gw::WebmailTemplate.new)
    end
    @default_template
  end

  def load_attachments(ref, item)
    if ref.has_attachments?
      ref.attachments.each do |f|
        file = Gw::WebmailMailAttachment.new({
          :tmp_id => item.tmp_id.to_s,
        })
        tmpfile = Sys::Lib::File::Tempfile.new({
          :data     => f.body,
          :filename => f.name
        })
        file.save_file(tmpfile) # pass the errors
      end
    end
  end

  def mdn_request_mode
    mdnRequest = :manual
    domain = Core.config['mail_domain']
    addrs = @item.disposition_notification_to_addrs
    begin
      if !domain.blank? && addrs && addrs.size > 0 &&
        addrs[0].address =~ /[@\.]#{Regexp.escape(domain)}$/i
        mdnRequest = :auto
      end
    rescue => e
      #Disposition-Notification-Toのパースエラー対策
      error_log(e)
      mdnRequest = nil
    end
    mdnRequest
  end

  def send_mdn_message(mdn_mode)
    mdn = Gw::WebmailMail.new
    mdn.in_from ||= %Q(#{Core.current_user.name} <#{Core.current_user.email}>)
    mail = mdn.prepare_mdn(@item, mdn_mode.to_s, request)
    mail.delivery_method(:smtp, ActionMailer::Base.smtp_settings)
    mail.deliver

    Core.imap.uid_store(@item.uid, "+FLAGS", "$Notified")
    @item.flags << "$Notified"
  end

  def load_address_from_flash
    @item.in_to  = flash[:mail_to] if flash[:mail_to]
    @item.in_cc  = flash[:mail_cc] if flash[:mail_cc]
    @item.in_bcc = flash[:mail_bcc] if flash[:mail_bcc]
    @item.in_subject = flash[:mail_subject] if flash[:mail_subject]
    @item.in_body = flash[:mail_body] if flash[:mail_body]
  end

  def concat_mail_body(quot_body, sign_body)
    sign_position = Gw::WebmailSetting.user_config_value(:sign_position)
    if sign_position.blank?
      "#{sign_body}#{quot_body}"
    else
      "#{quot_body}#{sign_body}"
    end
  end

  def get_sort_params(mailbox_name)
    key = params[:sort_key]
    order = params[:sort_order]

    unless key
      key = 'date'
      order = 'reverse'
    end

    params[:sort_key] = key
    params[:sort_order] = order

    sort_params = []
    sort_params += [order.upcase] if order.upcase != ""
    sort_params += [key.upcase]
    sort_params += ['REVERSE', 'DATE'] if key != 'date'
    sort_params
  end

  def make_search_params
    s_params = params.dup
    [:controller, :action, :mailbox].each {|n| s_params.delete(n) }
    s_params
  end

  def make_search_filter
    filter = @filter
    if params[:search]
      if !params[:s_status].blank?
        filter += ["UNSEEN"] if params[:s_status] == "unseen"
        filter += ["SEEN"]   if params[:s_status] == "seen"
      end
      if !params[:s_column].blank? && !params[:s_keyword].strip.blank?
        params[:s_keyword].gsub("　", " ").split(/ +/).each do |w|
          next if w.strip.blank?
          filter << "#{params[:s_column].upcase}"
          filter << "\"#{w.gsub('"', '\\"')}\""
        end
      end
      if !params[:s_label].blank?
        filter += ["KEYWORD $label#{params[:s_label]}"]
      end

      if params[:start_date].present?
        begin
          filter += ['SENTSINCE', Net::IMAP.format_date(Date.parse(params[:start_date]))]
        rescue => e
          flash.now[:error] = I18n.t("helpers.message.error_date")
        end
      end
      if params[:end_date].present?
        begin
          filter += ['SENTBEFORE', Net::IMAP.format_date(Date.parse(params[:end_date]) + 1)]
        rescue => e
          flash.now[:error] = I18n.t("helpers.message.error_date")
        end
      end
      if params[:received_start_date].present?
        begin
          filter += ['SINCE', Net::IMAP.format_date(Date.parse(params[:received_start_date]))]
        rescue => e
          flash.now[:error] = I18n.t("helpers.message.error_date")
        end
      end
      if params[:received_end_date].present?
        begin
          filter += ['BEFORE', Net::IMAP.format_date(Date.parse(params[:received_end_date]) + 1)]
        rescue => e
          flash.now[:error] = I18n.t("helpers.message.error_date")
        end
      end

    end

    if !params[:s_flag].blank?
      filter += ["FLAGGED"]   if params[:s_flag] == "starred"
      filter += ["UNFLAGGED"] if params[:s_flag] == "unstarred"
    end

    if params[:s_from]
      begin
        from_addr = nil
        from = Gw::WebmailMail.new.parse_address(params[:s_from]).first
        from_addr = from.address if from
      rescue => ex
        from_addr = nil
      end
      field = @mailbox.name =~ /^(Sent|Drafts)(\.|$)/ ? "TO" : "FROM"
      filter += [field, "\"#{from_addr.gsub(/"/, '')}\""] if from_addr
    end

    filter.join(' ')
  end

  def unique_filenames(filenames)
    uniques = []
    filenames.each do |filename|
      if uniques.include?(filename)
        for n in 1..255
          seq_filename = "#{File.basename(filename, ".*")}_#{n}#{File.extname(filename)}"
          unless uniques.include?(seq_filename)
            filename = seq_filename
            break
          end
        end
      end
      uniques << filename
    end
    return uniques
  end

  def parse_mailto(uri)
    mailto_params = Hash.new
    if uri
      begin
        [:to, :cc, :bcc, :subject, :body].each{|k| uri += "&#{k}=#{params[k]}" if params[k]}
        mailto = URI.parse(URI.escape(NKF::nkf('-w', uri)))
        if mailto.is_a?(URI::MailTo)
          mailto_params[:to] = URI.unescape(mailto.to)
          mailto.headers.each do |header|
            mailto_params[header[0].to_sym] = URI.unescape(header[1])
          end
        end
      rescue
      end
    end
    mailto_params
  end

  def load_address_histories(history_count)
    histories = Gw::WebmailMailAddressHistory.select('count(*) as cnt, address, friendly_address').where(:user_id => Core.current_user.id).group(:address).limit(history_count).order('cnt DESC, created_at DESC')

    emails = histories.map{|x| x.address}
    domain = Core.config.present? ? Core.config['mail_domain'] : '' #Core.configエラー回避
    sys_emails = histories.select{|x| x.address =~ /[@\.]#{Regexp.escape(domain)}$/i}.map{|x| x.address}

    addresses = Gw::WebmailAddress.select("email, name").where(:email => emails, :user_id => Core.current_user.id)
    sys_addresses = Sys::User.select("email, name").where(:email => sys_emails)

    histories.each do |history|
      if addr = addresses.find{|x| x.email == history.address}
        history.display_name = addr.name
      elsif addr = sys_addresses.find{|x| x.email == history.address}
        history.display_name = addr.name
      end
    end

    histories
  end

  def get_mailbox_uids(mailbox, uids)
    mailbox_uids = {}
    uids = [uids] if uids.is_a?(Fixnum)
    if mailbox.star_box?
      nodes = Gw::WebmailMailNode.find_nodes(mailbox.name, uids)
      nodes = nodes.select{|x| x.ref_mailbox && x.ref_uid}.group_by(&:ref_mailbox)
      nodes.each do |k,v|
        mailbox_uids[k] = v.map{|x| x.ref_uid}
      end
    else
      nodes = Gw::WebmailMailNode.find_ref_nodes(mailbox.name, uids)
      nodes = nodes.group_by(&:mailbox)
      nodes.each do |k,v|
        mailbox_uids[k] = v.map{|x| x.uid}
      end
    end
    mailbox_uids[mailbox.name] = uids
    mailbox_uids
  end

  def rescue_mail(e)
    @mailbox = Gw::WebmailMailbox.where(:user_id => Core.current_user.id, :name => params[:mailbox] || 'INBOX').first
    raise e unless @mailbox

    if params[:mobile] && params[:mobile].is_a?(Hash)
      action = params[:mobile][:form_action]
    else
      action = params[:action]
    end

    case action
    when 'create'
      @form_action = "create"
    when 'update'
      @form_action = "update"
      @form_method = "put"
    when 'answer'
      @form_action = "answer"
    when 'forward'
      @form_action = "forward"
    when 'resend'
      @form_action = "create"
    else
      raise e
    end

    @item = Gw::WebmailMail.new(params[:item])
    
    flash.now[:error] = I18n.t("helpers.message.occur_error") + "#{e}"
    render :action => :new
  end

private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    @title = I18n.t('rumi.mail.title.mail')
  end
end