# encoding: utf-8
class Gw::Admin::Webmail::SettingsController < Gw::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  helper Gw::MailHelper
  layout "admin/gw/webmail"
  before_action :set_title

  def pre_dispatch
    return redirect_to(request.env['PATH_INFO']) if params[:reset]
  end

  def index
    @all_webmail_setting = Sys::AllWebmailSetting.first
    @categories = Gw::WebmailSetting.user_categorized_configs
    _index @categories
  end

  def show
    @item = Gw::WebmailSetting.user_config(params[:id])
    _show @item
  end

  def new
    http_error(404)
  end

  def create
    http_error(404)
  end

  def update
    @item = Gw::WebmailSetting.user_config(params[:id])
    if params[:category] == 'switch_user'
      account = Sys::User.where(code: params[:account]).first.try(:email)
      raise unless Sys::Lib::Net::Imap.exists?(account, params[:password])
    end
    @item.set_value(params)
    _update(@item) do
    end
  rescue => e
    @item.errors.add(:base, I18n.t('rumi.config_settings.mail.errors.message.switch_user.authenticate_failed')) if params[:category] == 'switch_user'
    render :edit
  end

  def recognize(item)
    _recognize(item) do
      if @item.state == 'recognized'
        send_recognition_success_mail(@item)
      elsif @recognition_type == 'with_admin'
        if item.recognition.recognized_all?(false)
          users = Sys::User.find_managers
          send_recognition_request_mail(@item, users)
        end
      end
    end
  end
  
  def destroy
    http_error(404)
  end
private
  #ヘッダのタイトルを設定するメソッド
  def set_title
    case params[:category]
    when 'mail_list'
      @title = I18n.t('rumi.mail.title.setting.mail_list')
    when 'mail_detail'
      @title = I18n.t('rumi.mail.title.setting.mail_detail')
    when 'mail_form'
      @title = I18n.t('rumi.mail.title.setting.mail_form')
    when 'mail_pop'
      @title = I18n.t('rumi.mail.title.setting.mail_pop')
    when 'mail_label'
      @title = I18n.t('rumi.mail.title.setting.mail_label')
    when 'address'
      @title = I18n.t('rumi.mail.title.setting.address')
    when 'public_address_book'
      @title = I18n.t('rumi.mail.title.setting.public_address_book')
    when 'switch_user'
      @title = I18n.t('rumi.mail.title.setting.switch_user')
    else
      @title = ''
    end
  end
end
