# encoding: utf-8
class Sys::Admin::SwitchUsersController < Sys::Controller::Admin::Base
  include Sys::Controller::Scaffold::Base
  require 'csv'

  def pre_dispatch
    return error_auth unless Core.user.has_auth?(:manager)
  end

  def index
  end

  def import
    if !params[:item] || !params[:item][:file]
      return redirect_to(:action => :index)
    end

    require 'nkf'
    csv = NKF.nkf('-w', params[:item][:file].read)

    results = import_switch_users(csv)

    messages = []
    messages << "-- " + I18n.t("rumi.attributes.add") + "#{results[0]}" + I18n.t("helpers.number.unit.addresses")
    messages << "-- " + I18n.t("rumi.attributes.delete") + "#{results[1]}" + I18n.t("helpers.number.unit.addresses") if results[1] != 0
    messages << "-- " + I18n.t("rumi.attributes.fail") + "#{results[2]}" + I18n.t("helpers.number.unit.addresses") if results[2] != 0

    flash[:notice] = I18n.t("helpers.message.import") + "<br />#{messages.join('<br />')}".html_safe
    return redirect_to(:action => :index)
  end

protected

  def import_switch_users(csv)
    results = [0, 0, 0]
    switch_user_count = Gw::WebmailSetting.switch_user_max_count

    CSV.parse(csv, :headers => true) do |data|
      account = data[I18n.t("rumi.setting.switch_user.user_id")]
      switch_user_account = []
      switch_user_password = []
      (0..switch_user_count).each do |i|
         switch_user_account << data[I18n.t("rumi.setting.switch_user.switch_dest") + "#{i}" + I18n.t("rumi.setting.switch_user.user_id")]
         switch_user_password << data[I18n.t("rumi.setting.switch_user.switch_dest") + "#{i}" + I18n.t("rumi.setting.switch_user.password")]
      end

      if account.blank?
        results[2] += 1
        next
      end

      unless user = Sys::User.find_by_code(account)
        results[2] += 1
        next
      end

      Gw::WebmailSetting.where(user_id: user_id)
                        .where("name LIKE ?", 'switch_user%').delete_all

      saved_count = 0
      (0..switch_user_count).each do |i|
        next if switch_user_account[i].blank?

        hash = {}
        hash[:account] = switch_user_account[i]
        hash[:password] = Util::String::Crypt.encrypt_with_mime(switch_user_password[i]) || ''

        setting = Gw::WebmailSetting.new
        setting.user_id = user.id
        setting.name    = "switch_user#{i}"
        setting.value   = hash.to_json
        if setting.save
          saved_count += 1
        end
      end

      if saved_count > 0
        results[0] += 1
      else
        results[1] += 1
      end
    end

    results
  end
end
